package top.ninwoo.utils.service.impl;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import top.ninwoo.utils.entity.BridgePort;
import top.ninwoo.utils.entity.Ovs;
import top.ninwoo.utils.entity.OvsBridge;
import top.ninwoo.utils.service.OVSService;
import top.ninwoo.utils.util.LinuxCtlUtils;
import top.ninwoo.utils.util.OvsUtils;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @description 拟采用带状态的实现方案，采用单独线程进行异步更新，无论外部请求多少
 *              始终限制在一个用户去查询
 * @author joliu
 */
@Service
public class OVSServiceImpl implements OVSService, InitializingBean {

    @Autowired
    private OvsUtils ovsUtils;

    // TODO: 感觉应该把这个OVS类拆分开
    private ConcurrentHashMap<String, OvsBridge> ovsBridges = new ConcurrentHashMap<>();
    // 是否安装了ovs
    private boolean isInstall = false;
    // ovs版本号
    private String ovsVersion;
    private String ovsId;

    @Override
    public void afterPropertiesSet() throws Exception {
        fullUpdate();
    }

    /**
     * 全量更新
     */
    public void fullUpdate() {
        isInstall = ovsUtils.isInstall();

        if(!isInstall) {
            throw new RuntimeException("Ovs is not install properly!");
        }
        // 更新ovsBridge
        Ovs ovs = ovsUtils.showDetails();
        for (OvsBridge bridge : ovs.getBridges()) {
            ovsBridges.put(bridge.getBridgeId(), bridge);
        }

        ovsVersion = ovs.getOvsVersion();
        ovsId = ovs.getId();
    }

    /**
     * 定时更新Ovs状态
     */
    @Scheduled(fixedRate = 500)
    public void updateOvsStatus() {
        Ovs ovs = ovsUtils.showDetails();
        // 第一次检查，判断Bridge列表是否减少
        if(!ovs.getId().equals(ovsId)) {
            // 清空原有的配置，进行一次全量更新
            fullUpdate();
        }
        // 第二次检查，判断是否有新的bridge添加
        List<OvsBridge> bridges = ovs.getBridges();
        Set<String> updateList = new HashSet<>();
        bridges.forEach(b -> {
            if(!ovsBridges.containsKey(b.getBridgeId())) {
                ovsBridges.put(b.getBridgeId(), b);
                updateList.add(b.getBridgeId());
            } else if(ovsBridges.containsKey(b.getBridgeId())
                    && !ovsBridges.get(b.getBridgeId()).equals(b)) {
                ovsBridges.put(b.getBridgeId(), b);
                updateList.add(b.getBridgeId());
            } else if(ovsBridges.containsKey(b.getBridgeId())
                    && ovsBridges.get(b.getBridgeId()).equals(b)) {
                updateList.add(b.getBridgeId());
            }
        });
        // 删除丢失的点
        ovsBridges.keySet().forEach(k -> {
            if(!updateList.contains(k)) {
                ovsBridges.remove(k);
            }
        });
    }

    /**
     * 获取ovs网桥列表
     * @return
     */
    @Override
    public List<OvsBridge> getOvsBridges() {
        return new ArrayList<>(ovsBridges.values());
    }
}
